#vider la mmoire
rm(list=ls())

#lancer le garbage collector
gc()

#pour mesurer le temps de calcul
library(tictoc)

#library parallel
library(parallel)

#packages combinaison de foreach et doParallel
library(foreach)
library(doParallel)

#information sur les versions
#sessionInfo()
#help(package='parallel')

#fonction pour calculer le min
#on passe par un tri pas trs finaud (par slection)
#https://fr.wikipedia.org/wiki/Tri_par_slection
#et on rcupre la premire valeur
#algorithme en O(n^2)
mon_min <- function(v) {
  #copie locale
  temp <- v
  #longueur du vecteur
  n <- length(temp)
  #tri par slection si (n > 1)
  if (n > 1) {
    #recherche des minimums successifs
    for (i in 1:(n - 1)) {
      i_mini <- i
      for (j in (i + 1):n) {
        if (temp[j] < temp[i_mini]) {
          i_mini <- j
        }
      }
      #changer
      if (i_mini != i) {
        tempo <- temp[i]
        temp[i] <- temp[i_mini]
        temp[i_mini] <- tempo
      }
    }
  }
  #la plus petite valeur est le min.
  return(temp[1])
}

#gnrer le vecteur de donnes
n <- 15000
a <- runif(n)

#appel de la fonction sur la totalit du vecteur
tic()
print(paste('Min direct =',mon_min(a)))
print('>> Temps de calcul - fonction mon_min direct')
toc()

#*************************
#travail avec **parallel**
#*************************

#pour mesurer le processus global de **parallel**
tic()

#nombre de blocs des donnes = nombre de cores
k <- 6

#partition en blocs des donnes
blocs <- split(a,1+(1:n)%%k)
#print(blocs)

#affichage nombre de coeurs dispo
#print(parallel::detectCores())

#configuration du nombre de core -> k
#plus ne sert  rien, moins c'est possible
#dmarrage des moteurs (workers)
clust <- parallel::makeCluster(k)

#lancement des min en parallle
res <- parallel::parSapply(clust,blocs,FUN = mon_min)

#rsultats intermdiaires
#print(res)

#fonction de consolidation
print(paste('Min parallel =',mon_min(res)))

#teindre les moteurs
parallel::stopCluster(clust)

#affichage temps de calcul
print('>> Temps de calcul total avec parSapply (split + min par bloc)')

#temps de calcul
toc()

#****************************************
#travail avec **foreach et **doParallel**
#****************************************

#pour mesurer le processus global de **foreach/doParallel**
tic()

#nombre de cores  exploiter
#k <- 6

#partition en blocs des donnes
blocs <- split(a,1+(1:n)%%k)
#print(blocs)

#configurer les cores
doParallel::registerDoParallel(k)

#itrer sur les blocs
res <- foreach::foreach(b = blocs, .combine = c) %dopar% {
  return(mon_min(b))
}

#rsultats intermdiaires
#print(res)

#minimum global
print(paste('Min foreach/dopar =',mon_min(res)))

#stopper les cores
doParallel::stopImplicitCluster()

#affichage temps de calcul
print('>> Temps de calcul total avec foreach/dopar (split + min par bloc)')

#temps de calcul
toc()
